@@ -11,19 +11,9 @@
 #include <QtQuick3DRuntimeRender/private/qssgrendershadercache_p.h>
 
 #if QT_CONFIG(opengl)
-#include <QtGui/private/qrhigles2_p.h>
 #include <QOffscreenSurface>
 #include <QOpenGLContext>
 #endif
-#if QT_CONFIG(vulkan)
-#include <QtGui/private/qrhivulkan_p.h>
-#endif
-#ifdef Q_OS_WIN
-#include <QtGui/private/qrhid3d11_p.h>
-#endif
-#if defined(Q_OS_MACOS) || defined(Q_OS_IOS)
-#include <QtGui/private/qrhimetal_p.h>
-#endif
 
 QT_BEGIN_NAMESPACE
 
@@ -55,17 +45,16 @@ void writeUInt32(QIODevice &device, quint32 value)
 
 void appendBinaryVector(QVector<char> &dest, const quint32 src)
 {
-    dest.reserve(dest.size() + sizeof(src));
-    for (size_t i = 0; i < sizeof(src); i++)
-        dest.push_back(reinterpret_cast<const char *>(&src)[i]);
+    qsizetype oldsize = dest.size();
+    dest.resize(dest.size() + sizeof(src));
+    memcpy(dest.data() + oldsize, &src, sizeof(src));
 }
 
 void appendBinaryVector(QVector<char> &dest, const std::string &src)
 {
-    dest.reserve(dest.size() + src.size() + 1);
-    for (auto c : src)
-        dest.push_back(c);
-    dest.push_back('\0');
+    qsizetype oldsize = dest.size();
+    dest.resize(dest.size() + src.size() + 1);
+    memcpy(dest.data() + oldsize, src.c_str(), src.size() + 1);
 }
 }
 
@@ -127,15 +116,15 @@ QString renderToKTXFileInternal(const char *name, const QString &inPath, const Q
     QRhiCommandBuffer *cb;
     rhi->beginOffscreenFrame(&cb);
 
-    const auto rhiContext = QSSGRef<QSSGRhiContext>(new QSSGRhiContext);
-    rhiContext->initialize(rhi.get());
-    rhiContext->setCommandBuffer(cb);
+    const auto rhiContext = std::make_unique<QSSGRhiContext>(rhi.get());
+    QSSGRhiContextPrivate *rhiCtxD = QSSGRhiContextPrivate::get(rhiContext.get());
+    rhiCtxD->setCommandBuffer(cb);
 
     QScopedPointer<QSSGLoadedTexture> inImage(QSSGLoadedTexture::loadHdrImage(QSSGInputUtil::getStreamForFile(inPath), FORMAT));
     if (!inImage)
         return QStringLiteral("Failed to load hdr file");
 
-    auto shaderCache = std::make_unique<QSSGShaderCache>(rhiContext);
+    auto shaderCache = std::make_unique<QSSGShaderCache>(*rhiContext);
 
     // The objective of this method is to take the equirectangular texture
     // provided by inImage and create a cubeMap that contains both pre-filtered
@@ -221,9 +210,9 @@ QString renderToKTXFileInternal(const char *name, const QString &inPath, const Q
         desc = { { 0,
                    0,
                    { inImage->textureFileData.data().constData() + inImage->textureFileData.dataOffset(0),
-                     int(inImage->textureFileData.dataLength(0)) } } };
+                     quint32(inImage->textureFileData.dataLength(0)) } } };
     } else {
-        desc = { { 0, 0, { inImage->data, int(inImage->dataSizeInBytes) } } };
+        desc = { { 0, 0, { inImage->data, inImage->dataSizeInBytes } } };
     }
     auto *rub = rhi->nextResourceUpdateBatch();
     rub->uploadTexture(sourceTexture, desc);
@@ -234,7 +223,7 @@ QString renderToKTXFileInternal(const char *name, const QString &inPath, const Q
     QRhiSampler *sampler = rhiContext->sampler(samplerDesc);
 
     // Load shader and setup render pipeline
-    QSSGRef<QSSGRhiShaderPipeline> envMapShaderStages = shaderCache->loadBuiltinForRhi("environmentmap");
+    const auto &envMapShaderStages = shaderCache->getBuiltInRhiShaders().getRhiEnvironmentmapShader();
 
     // Vertex Buffer - Just a single cube that will be viewed from inside
     QRhiBuffer *vertexBuffer = rhi->newBuffer(QRhiBuffer::Immutable, QRhiBuffer::VertexBuffer, sizeof(cube));
@@ -313,7 +302,7 @@ QString renderToKTXFileInternal(const char *name, const QString &inPath, const Q
     cb->resourceUpdate(rub);
 
     for (int face = 0; face < 6; ++face) {
-        cb->beginPass(renderTargets[face], QColor(0, 0, 0, 1), { 1.0f, 0 }, nullptr, QSSGRhiContext::commonPassFlags());
+        cb->beginPass(renderTargets[face], QColor(0, 0, 0, 1), { 1.0f, 0 }, nullptr, rhiContext->commonPassFlags());
 
         // Execute render pass
         cb->setGraphicsPipeline(envMapPipeline);
@@ -379,11 +368,7 @@ QString renderToKTXFileInternal(const char *name, const QString &inPath, const Q
     }
 
     // Load the prefilter shader stages
-    QSSGRef<QSSGRhiShaderPipeline> prefilterShaderStages;
-    if (isRGBE)
-        prefilterShaderStages = shaderCache->loadBuiltinForRhi("environmentmapprefilter_rgbe");
-    else
-        prefilterShaderStages = shaderCache->loadBuiltinForRhi("environmentmapprefilter");
+    const auto &prefilterShaderStages = shaderCache->getBuiltInRhiShaders().getRhienvironmentmapPreFilterShader(isRGBE);
 
     // Create a new Sampler
     const QSSGRhiSamplerDescription samplerMipMapDesc {
@@ -461,7 +446,7 @@ QString renderToKTXFileInternal(const char *name, const QString &inPath, const Q
     // Render
     for (int mipLevel = 0; mipLevel < mipmapCount; ++mipLevel) {
         for (int face = 0; face < 6; ++face) {
-            cb->beginPass(renderTargetsMap[mipLevel][face], QColor(0, 0, 0, 1), { 1.0f, 0 }, nullptr, QSSGRhiContext::commonPassFlags());
+            cb->beginPass(renderTargetsMap[mipLevel][face], QColor(0, 0, 0, 1), { 1.0f, 0 }, nullptr, rhiContext->commonPassFlags());
             cb->setGraphicsPipeline(prefilterPipeline);
             cb->setVertexInput(0, 1, &vbufBinding);
             cb->setViewport(QRhiViewport(0, 0, mipLevelSizes[mipLevel].width(), mipLevelSizes[mipLevel].height()));
@@ -489,21 +474,18 @@ QString renderToKTXFileInternal(const char *name, const QString &inPath, const Q
 
     // Prepare Key/Value array
     {
-        std::map<std::string, std::string> keyValueList;
+        // Add a key to the metadata to know it was created by our IBL baker
+        static const char key[] = "QT_IBL_BAKER_VERSION";
+        static const char value[] = "1";
 
-        // Add a key to the metadata to know it was created by our IBL baker and what version was used.
-        keyValueList["QT_IBL_BAKER_VERSION"] = "1";
+        constexpr size_t keyAndValueByteSize = sizeof(key) + sizeof(value);   // NB: 2x null terminator
+        appendBinaryVector(keyValueData, keyAndValueByteSize);
+        appendBinaryVector(keyValueData, key);
+        appendBinaryVector(keyValueData, value);
 
-        for (auto &kv : keyValueList) {
-            quint32 keyAndValueByteSize = quint32(kv.first.size() + kv.second.size() + 2); // NB: 2x null terminator
-            appendBinaryVector(keyValueData, keyAndValueByteSize);
-            appendBinaryVector(keyValueData, kv.first);
-            appendBinaryVector(keyValueData, kv.second);
-
-            const quint32 padding = 3 - ((keyAndValueByteSize + 3) % 4); // Pad until next multiple of 4
-            for (quint32 i = 0; i < padding; i++)
-                keyValueData.push_back(char(0x00));
-        }
+        // Pad until next multiple of 4
+        const size_t padding = 3 - ((keyAndValueByteSize + 3) % 4); // Pad until next multiple of 4
+        keyValueData.resize(keyValueData.size() + padding);
     }
 
     // Header
@@ -627,6 +609,8 @@ QRhi::Implementation getRhiImplementation()
         implementation = QRhi::OpenGLES2;
     } else if (rhiBackend == QByteArrayLiteral("d3d11") || rhiBackend == QByteArrayLiteral("d3d")) {
         implementation = QRhi::D3D11;
+    } else if (rhiBackend == QByteArrayLiteral("d3d12")) {
+        implementation = QRhi::D3D12;
     } else if (rhiBackend == QByteArrayLiteral("vulkan")) {
         implementation = QRhi::Vulkan;
     } else if (rhiBackend == QByteArrayLiteral("metal")) {
@@ -669,7 +653,9 @@ QString renderToKTXFile(const QString &inPath, const QString &outPath)
             params.format.setVersion(3, 0);
         }
         params.fallbackSurface = QRhiGles2InitParams::newFallbackSurface();
-        return renderToKTXFileInternal("OpenGL", inPath, outPath, QRhi::OpenGLES2, &params);
+        const QString result = renderToKTXFileInternal("OpenGL", inPath, outPath, QRhi::OpenGLES2, &params);
+        delete params.fallbackSurface;
+        return result;
     }
 #endif
 
@@ -687,10 +673,13 @@ QString renderToKTXFile(const QString &inPath, const QString &outPath)
     if (rhiImplementation == QRhi::D3D11) {
         QRhiD3D11InitParams params;
         return renderToKTXFileInternal("Direct3D 11", inPath, outPath, QRhi::D3D11, &params);
+    } else if (rhiImplementation == QRhi::D3D12) {
+        QRhiD3D12InitParams params;
+        return renderToKTXFileInternal("Direct3D 12", inPath, outPath, QRhi::D3D12, &params);
     }
 #endif
 
-#if defined(Q_OS_MACOS) || defined(Q_OS_IOS)
+#if QT_CONFIG(metal)
     if (rhiImplementation == QRhi::Metal) {
         QRhiMetalInitParams params;
         return renderToKTXFileInternal("Metal", inPath, outPath, QRhi::Metal, &params);
