@@ -1050,7 +1050,6 @@ void QQuick3DModel::onMorphTargetDestroyed(QObject *object)
     }
     if (found) {
         markDirty(QQuick3DModel::MorphTargetsDirty);
-        m_numMorphAttribs = 0;
     }
 }
 
@@ -1059,14 +1058,7 @@ void QQuick3DModel::qmlAppendMorphTarget(QQmlListProperty<QQuick3DMorphTarget> *
     if (morphTarget == nullptr)
         return;
     QQuick3DModel *self = static_cast<QQuick3DModel *>(list->object);
-    if (self->m_numMorphAttribs >= 8) {
-        qWarning("The number of morph attributes exceeds 8. This morph target will be ignored.");
-        return;
-    }
     self->m_morphTargets.push_back(morphTarget);
-    self->m_numMorphAttribs += morphTarget->numAttribs();
-    if (self->m_numMorphAttribs > 8)
-        qWarning("The number of morph attributes exceeds 8. This morph target will be supported partially.");
 
     self->markDirty(QQuick3DModel::MorphTargetsDirty);
 
@@ -1113,7 +1105,6 @@ void QQuick3DModel::qmlClearMorphTargets(QQmlListProperty<QQuick3DMorphTarget> *
         disconnect(morph, &QQuick3DMorphTarget::destroyed, self, &QQuick3DModel::onMorphTargetDestroyed);
     }
     self->m_morphTargets.clear();
-    self->m_numMorphAttribs = 0;
     self->markDirty(QQuick3DModel::MorphTargetsDirty);
 }
 
@@ -1179,10 +1170,10 @@ void QQuick3DModel::setLevelOfDetailBias(float newLevelOfDetailBias)
     \default 0.0
 
     A value greater than zero means this value will override the
-    SceneEnvironment::texelsPerUnit value for this specific model during
+    \l {Lightmapper::texelsPerUnit} value for this specific model during
     lightmap baking.
 
-    \sa SceneEnvironment::texelsPerUnit
+    \sa Lightmapper::texelsPerUnit
 */
 
 float QQuick3DModel::texelsPerUnit() const
